\name{COMError}
\alias{COMError}
\alias{getCOMErrorCode}
\alias{.COMErrors}
\title{Constructor for COM error}
\description{
This is an S-level constructor
that allows a programmer to return an error from
an S function to a C-level routine.
One can indicate different types of errors
using the different error codes
that are available in the S object named \code{.COMErrors}.
}
\usage{
COMError(code, description = "", className = "COMError")
getCOMErrorCode(name)
}
\arguments{
  \item{code}{a name or value of the error code that corresponds to the internal
    C-level API for Windows errors. These are dynamically computed 
    when the package is loaded and available as an S vector
    as in the variable \code{.COMErrors} }
  \item{description}{text describing the problem.}
  \item{className}{the name of the S3 class used to represent 
    the error. Currently, this is just an S3-style class name.}
  \item{name}{the name of the COM error whose numeric value is to be retrieved. }
}
\value{
 An S3-style object containing elements
for 
 \item{code}{the numeric value of the code}
 \item{description}{the string describing the error}
}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/SWinRegistry}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{SCOMIDispatch}}
}
\examples{
 COMError("DISP_E_MEMBERNOTFOUND", "No such element 'foo'")

 name <- "foo"
 COMError("DISP_E_MEMBERNOTFOUND", paste("No such element '", name, "'", sep=""))
}
\keyword{interface}

\eof
\name{COMNameManager}
\alias{COMNameManager}
\title{Generator function for name-id managing cache}
\description{

 This is a constructor function that generates a function object which
can be used to manage a unique set of name-identier pairs.  Each
resulting function from a call to this generator stores its own set
map of the names and their integer identifiers.  This can be used when
managing the mapping of COM names for properties, methods and
parameters. The behavior of the manager function is to allow
any names to be requested and simply add previously unseen 
values to the map. In this way, it is non-discriminating.
}
\usage{
COMNameManager(map = integer(0))
}
\arguments{
  \item{map}{a named integer vector giving an initial value
   for the mapping of the names to integers and vice-versa.}
}
\details{
  This generator function is merely used to 
 create a unique environment for each instance
 of the return value.
}
\value{
 A single function which acts as a name manager.
 If called with a numeric vector, this function returns 
 the associated names in its local map.
 If called with a character vector, it looks up the
 names and returns their associated integer identifiers.
 Any values not already in the map are added.
 And if called with no argument, the current map is returned.
}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}

\seealso{
\code{\link{SCOMNamedFunctionClass-class}}
 \code{\link{SCOMNamedFunctionClass}}
 \code{\link{SCOMNamedFunctionDispatch}}
}
\examples{

}
\keyword{interface}

\eof
\name{COMSIDispatchObject}
\alias{COMSIDispatchObject}
\title{S-level IDispatch Mechanism for S-COM}
\description{
 This function is closely associated with
 the \code{\link{SCOMEnvironmentClass-class}}
 and \code{\link{SCOMFunctionClass}}
 classes and provides the necessary 
 functionality to control the
 COM invocation of methods and accessing of
 properties for an S-COM object.
 This provides S functions that implement
 the \code{Invoke} and \code{GetIDsOfNames}
 methods for the COM object.
 The \code{GetIDsOfNames} maps COM names
 to unique identifiers and \code{Invoke}
 handles the accessing of methods and properties
 in the S-level object underlying the S-COM object.
}
\usage{
COMSIDispatchObject(funs, properties = character(0),
	             .propertyEnvironment = environment(funs[sapply(funs, is.function)][[1]]))
}
\arguments{
  \item{funs}{a named list of the functions that are to be used as methods,
    and a .properties elements that give the names of the variables
   within \code{.propertyEnvironment} that are to be treated as properties.}
  \item{properties}{
   a character vector giving the names of the variables  in the 
   environment which are to be treated as properties of the COM object.
   This is used if it is supplied and there is no \code{.properties}
   element in \code{funs}.}
  \item{.propertyEnvironment}{the environment in which to find the properties.
   By default, this is taken as the environment of the first function in the list
   of methods, i.e. \code{funs}.}
}

\details{
 This computes the list of all names in the
 properties, and  functions and their parameters.
 (This means that named values in \code{...} are currently
  not permitted, however this is easy to change.)
 and computes a map to use for associating names and identifiers.
  These are used by the client to map a name to an identifier and
 use that in calls.
}

\value{
 A list of two functions 
 that can be passed to the constructor
 of the C++ class \code{RCOMSObject}
 via the C routine \code{R_RCOMSObject}.
 The functions are the \code{Invoke} and \code{GetIDsOfNames}.
}
\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/SWinRegistry}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{computeCOMNames}}
}

\examples{

}
\keyword{interface}

\eof
\name{SCOMNamedFunctionDispatch}
\alias{SCOMNamedFunctionDispatch}
\title{Create COM dispatch mechanism for COMSNamedFunctionClass instance}
\description{

 This function is used to create a dispatch mechanism for an instance
of an S COM object created from a \code{\link{SCOMNamedFunctionClass-class}}
definition object. This provides an invocation mechanism to handle
calls to the methods in the COM object and also a mapping of the names
of methods, properties and parameters to integers and back.

}
\usage{
SCOMNamedFunctionDispatch(funs, properties, propertyNames, nameMgr = COMNameManager())
}
\arguments{
  \item{funs}{a named character vector.
   The names are those used by COM clients
   to refer to methods in the COM server.
   The correspoding element in the vector
   is the name of the S function to invoke
   for that method name.
  }
  \item{properties}{a list of named values that act as local properties
   for the instance of the COM server}
  \item{propertyNames}{a character vector giving the names of 
    S variables that act as additional global or shared properties
    for the COM objects.}
  \item{nameMgr}{a function that performs all of the name-identifer mapping
  for properties. This function should work on numbers and return the corresponding names,
   names and return the corresponding identifiers, and if called with no arguments
   return the current map as a named integer vector.
   See \code{\link{COMNameManager}}. This is made accessible as an argument
   primarily so that one can provide a
   \code{\link{COMNameManager}} function that is initialized
   with a particular mapping, say computed from a type library.}
}

\value{
 A list with two functions that act as the
 invocation and name manager for the COM dispatch mechanism.
 \item{Invoke}{the invocation function that handles calls to 
   methods and properties by COM clients.}
 \item{GetNamesOfIDs}{the name mapper that is called by the client
  to map a name to an integer identifier.}
}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{SCOMNamedFunctionClass-class}}
 \code{\link{SCOMNamedFunctionClass}}
 \code{\link{createCOMObject}}
}
\examples{

}
\keyword{interface}
\keyword{programming}


\eof
\name{GetCOMClassDef}
\alias{GetCOMClassDef}
\title{Find S COM class definition from a class UUID}
\description{
  This function is called by a COM factory  to create an
  instance of a particular COM class defined in S when
  such an object is requested by a client.
  This looks for the information to create the COM
  object in two ways.

  Firstly, it attempts to look in the registry for
  the class entry for the given UUID identifier.
  If this exists, it looks for entries for
  \code{profile} to see if there is an S script
  that it should execute before creating the server.
  This can be used to initialize the R session appropriately
  so that packages, methods, data, etc. are available to support
  the server.
  It then checks to see if there is an entry in the registry
  named \code{rda} and if so, deserializes the file given by
  that entries value. The first object from deserializing is
  used as the COM server object.
  If no \code{rda} entry exists but there is an entry
  named \code{Svariable}, we then attempt to retrieve
  the S object that is currently bound/assigned to that variable
  and use that.
  
  The typical mechanism does not use \code{rda} or \code{Svariable}
  entries. Instead, the developer of a COM server in S
  registers the server definition using
  \code{\link{registerCOMClass}}.  \code{GetCOMClassDef}
  then uses the UUID passed to it to find the definition
  in a list of such server definitions. This list
  is stored in a serialized S object residing on the system
  and created during the call to \code{\link{registerCOMClass}}.
  
}
\usage{
GetCOMClassDef(clsid, rda = getCOMClassDefFileName(), verbose = FALSE)
}
\arguments{
  \item{clsid}{the UUID identifying the particular COM class definition.
   This is an object of class uuid from the \code{Ruuid} package
   or a character vector giving the UUID in string form.}  
  \item{rda}{the name of an R data file containing the serialized object
    giving the list of COM class definitions. 
  } 
  \item{verbose}{a logical value indicating whether to write diagnostic messages about progress
   to the console.}  
}
\value{
  An object that "defines" an S COM server
  which can be passed to \code{\link{createCOMObject}}.
}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/SWinRegistry}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
\code{\link{registerCOMClass}}  
\code{\link{createCOMObject}}
}
\examples{

}
\keyword{interface}

\eof
\name{SCOMClass-class}
\docType{class}
\alias{SCOMClass-class}
\title{Class "SCOMClass" ~~~ }
\description{
This is the top-level virtual class for representing
the basic S-language definition for a COM object.
It provides slots for the name, class identifier (UUID),
the version identifier (string), the
program identifier and a string giving a description
of the class.
This is intended to be sub-classed by other particular
COM definition classes in S which provide a definition
for how to create instances of the object in S
that can be used as COM objects.
}
\section{Objects from the Class}{
 This is a virtual class.
}
\section{Slots}{
  \describe{
    \item{\code{name}:}{Object of class \code{"character"}. A string  
    giving the human-usable name for the COM class. This is used
    by clients to create an instance of the class.}
    \item{\code{version}:}{Object of class \code{"character"}.
     A string giving information about the version of the class.
     This is a free formatted string at present, but should be something of the form
     "major.minor-patch".}
    \item{\code{classId}:}{Object of class \code{"Ruuid"} 
     a unique identifier for the class. One can use 
     the \code{Ruuid} package from \url{http://www.bioconductor.org}
     or a system tool such as \code{guidgen} to generate the UUID.
     You will need the \code{Ruuid} package to create the object however.
     }
    \item{\code{progId}:}{Object of class \code{"Ruuid"} 
     not currently used. }
    \item{\code{help}:}{Object of class \code{"character"} 
       a string giving a human-readable description of the class.}
  }
}

\section{Methods}{
No methods defined with class "SCOMClass" in the signature.
}


\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/SWinRegistry}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}

\seealso{
 \code{\link{SCOMEnvironmentClass-class}}
 \code{\link{SCOMFunctionClass-class}}
 \code{\link{SCOMIDispatch-class}}
 \code{\link{SCOMNamedFunctionClass-class}}
}

\examples{
}
\keyword{classes}
\eof
\name{SCOMEnvironmentClass-class}
\docType{class}
\alias{SCOMEnvironmentClass-class}
\title{Class for COM definitions involving a generator function, typically using closures.}
\description{

This class is used to represent a COM definition that involves a
generator function that is called to create unique instances when a
COM object is needed of this class.  The part of the class that
involves an \code{environment} is typically that the generator
function returns a list of functions that share a common environment
and the variables within that environment.  This gives the mutable
state and this is a very natural way to think of COM objects in R.

This class should not be used directly at present. Instead,
use \code{SCOMIDispatch-class} which is almost exactly
the same but uses a more complete dispatch mechanism that is
implemented in S (\code{\link{COMSIDispatchObject}}).

}
\section{Objects from the Class}{
 Using this class directly is stronly discouraged at present.
 Instead, use the \code{\link{SCOMIDispatch-class}}
 and its constructor function \code{\link{SCOMIDispatch}}.
}
\section{Slots}{
  \describe{
    \item{\code{generator}:}{Object of class \code{"function"}.
     This is the generator function that is invoked each time
     a new COM object is requested for this class.
     It is expected to return a list of functions which are used
     as the methods. The names of the elements in the list are used
     as the names of the methods that are visible to clients.
     Any properties that are to be exported are identified
     by having an element named \code{.properties}
     in the list which contains a character vector giving
     the names of the accessible properties.
    }
  }
}
\section{Extends}{
Class \code{"SCOMClass"}, directly.
}
\section{Methods}{
  \describe{
    \item{createCOMObject}{\code{signature(def = "SCOMEnvironmentClass")}: ... }
  }
}
\references{
 \url{http://www.omegahat.org/RDCOMServer}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{SCOMIDispatch}}
}

\examples{
# Don't use this class directly. 
# 


}
\keyword{classes}
\eof
\name{SCOMFunctionClass-class}
\docType{class}
\alias{SCOMFunctionClass-class}
\title{COM definition using named function list}
\description{
This class allows one to define different COM classes by grouping S
function objects together that act as methods.  These functions can
but need not share an environment.  The key thing is that they are
shared across different instances of the COM class, unlike
\code{SCOMEnvironmentClass-class} servers which have different
instances of the functions and their environments for each object.

 Unlike the \code{\link{SCOMIDispatch}} constructor,
 this definition does not have a generator function
 but instead re-uses the list of functions for each
 COM object that is created for this  class.
 In this way, they cannot share state.

 The properties are currently accessed as variables in
 the global environment and are specified simply as variable
 names. A variant of this class will allow them to be local to
 the object and specified as a list.

}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("SCOMFunctionClass", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{functions}:}{Object of class \code{"list"}
   this is a named list of functions. The names are used to identify the
   methods when dispatching the invocation by COM clients.
   The methods are implemented by the particular functions.
   The functions can share an environment, however there is no generator
   function to create separate instances of these functions for each COM
   object. Thus the environment would be shared across all objects, i.e. static
   methods.}
    \item{\code{properties}:}{Object of class \code{"character"}
       this is a vector giving the names of the top-level/global
     variables that are to be made accessible as properties
     of the COM server.
   }
  }
}
\section{Extends}{
Class \code{"SCOMClass"}, directly.
}
\section{Methods}{
  \describe{
    \item{createCOMObject}{\code{signature(def = "SCOMFunctionClass")}:
     a method that creates an instance of this server
     as a C++ object along with a dispatch function to invoke the
     methods in the \code{functions} list.  }
  }
}
\references{
 \url{http://www.omegahat.org/RDCOMServer}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{createCOMObject}}
 \code{\link{SCOMEnvironmentClass-class}}
 \code{\link{SCOMNamedFunctionClass-class}}
}

\examples{

}
\keyword{classes}
\eof
\name{SCOMFunctionClass}
\alias{SCOMFunctionClass}
\title{Constructor for Simple S-COM definition}
\description{
 This is the constructor function for defining a 
 COM  class in S that consists of a simple list
 of functions and properties that do not share
 state.

 Unlike the \code{\link{SCOMIDispatch}} constructor,
 this definition does not have a generator function
 but instead re-uses the list of functions for each
 COM object that is created for this  class.
 In this way, they cannot share state.

 The properties are currently accessed as variables in
 the global environment and are specified simply as variable
 names. A variant of this class will allow them to be local to
 the object and specified as a list.
}
\usage{
SCOMFunctionClass(functions, properties = character(0), name, help = "", def = new("SCOMFunctionClass"), where = "")
}

\arguments{
  \item{functions}{a named list of functions. The names are the externally accessible 
   method names by which clients access functionality of objects of these classes.}
  \item{properties}{currently a character vector giving the names of S variables in
   the search path that can be accessed by clients.}
  \item{name}{the human-usable name of the COM class}
  \item{help}{a character vector giving description strings for each of the 
    methods (and properties).  Currently not used but will be in the future.}
  \item{def}{an instance of the target S class that is used to represent this definition.
   This is available as an argument to allow constructors for sub-classes to call
   this and supply their own instance to be populated with values.
   }
  \item{where}{currently unused but here for automatic registration.}
}
\details{

}
\value{
 The \code{def} object with slots filled in.

}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/SWinRegistry}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{SCOMIDispatch}}
}

\examples{

}
\keyword{interface}

\eof
\name{SCOMNamedFunctionClass}
\docType{class}
\alias{SCOMNamedFunctionClass-class}
\title{COM definition using global S functions}
\description{

COM objects created from this class offer methods that are simply S
functions.  Each COM class of this type provides access to a
collection of (related) S functions and values.  In addition to
accessing these global functions, each instance can have additional
properties that are specific to it and not shared across different COM
servers.

}
\section{Objects from the Class}{
}
\section{Slots}{
  \describe{
    \item{\code{functions}:}{Object of class \code{"character"} 
       a character vector giving the names of the
      S global functions which are to be made available as 
      methods for the COM server.
    }
    \item{\code{propertyNames}:}{Object of class \code{"character"} 
     the names of S global variables which are to be made accessible as 
    properties of the COM server. }
    \item{\code{properties}:}{Object of class \code{"list"} 
      additional properties given as a named list of values.
      These are per-instance values that are separate across different 
      instances of this server class.
    }
  }
}
\section{Extends}{
Class \code{"SCOMClass"}, directly.
}
\section{Methods}{
  \describe{
    \item{createCOMObject}{\code{signature(def = "SCOMNamedFunctionClass")}: 
    creates an instance of this server as a C++-level object.}
  }
}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}

\seealso{
\code{\link{createCOMObject}}
 \code{\link{SCOMEnvironmentClass-class}}
 \code{\link{SCOMFunctionClass-class}}
}

\examples{

}
\keyword{classes}
\eof
\name{SCOMIDispatch-class}
\docType{class}
\alias{SCOMIDispatch-class}
\title{Typical COM server class representation}
\description{
This class is a simple extension
of \code{SCOMEnvironmentClass-class} and so it
supports having a generator function
that is used to create each instance of
the COM object at the S level (e.g. 
creating unique function lists for each instance
that share a common environment with each other
but separate from other instances).

The only difference between this and \code{SCOMEnvironmentClass-class}
is that we use a different dispatch mechanism for the two classes.
For this class, we use the more complete and easier-to-understand S
function \code{\link{COMSIDispatchObject}} to create both the dispatch
mechanism that handles method invocation for the object, and the
mapping of names to integers and back.  The
\code{SCOMEnvironmentClass-class} uses a C++-level mechanism and
should not be used at this point.

}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("SCOMIDispatch", ...)}.
}
\section{Slots}{
  \describe{
  }
}
\section{Extends}{
Class \code{"SCOMEnvironmentClass"}, directly.
Class \code{"SCOMClass"}, by class "SCOMEnvironmentClass".
}
\section{Methods}{
  \describe{
    \item{createCOMObject}{\code{signature(def = "SCOMIDispatch")}: 
     create an instance of the COM server and the associated C++ object
     that makes the server available to clients.}
  }
}
\references{
 \url{http://www.omegahat.org/RDCOMServer}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}

\seealso{
\code{\link{createCOMObject}}
\code{\link{SCOMIDispatch}}
\code{\link{SCOMEnvironmentClass-class}}
}

\examples{
\dontrun{
 gen = function(mu = 0, sd = 1) {
         list(sample=function(n) rnorm(n, mu, sd),
              quantiles=function(p) qnorm(p, mu, sd)
             )
       }

 def = SCOMIDispatch(gen, "SimpleNormal")
 registerCOMClass(def)
}
}
\keyword{classes}
\eof
\name{SCOMIDispatch}
\alias{SCOMIDispatch}
\title{Define S-level COM class}
\description{
 This defines an S-level COM object
 in terms of a generator function that,
 when called, creates a list of functions
 and meta-information about properties that
 represents a unique object whose methods
 are the functions and whose properties are 
 identified by a \code{.properties} element
 within the list.
 This object is then used in conjunction 
 with \code{\link{createCOMObject}} to 
 provide an invokable COM object.
}
\usage{
SCOMIDispatch(generator, name, help = "", def = new("SCOMIDispatch"), where = "")
}
\arguments{
  \item{generator}{the generator function which is called to create each individual
  object. This should return a named list of functions
  and property identifiers. The properties are identified by 
  a character vector in the element of the returned list named
  \code{.properties}.}
  \item{name}{the user-friendly name of the COM class which clients can
   use to create an instance of the COM object.}
  \item{help}{a string providing a description of the COM class.}
  \item{def}{a COM object which is filled in by this function
   and the \code{.initSCOMClass} function.
   This allows us to use inheritance relatively easily within these constructor
   functions by creating an object of a derived class and passing it up 
   to the  constructor functions for the ancestor classes.}
  \item{where}{ignored at present, but could be used to register the class.}
}
\details{

}
\value{
 An object of class \code{SCOMIDispatchClass}.
}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{SCOMFunctionClass}}
 \code{\link{createCOMObject}}
}

\examples{

}
\keyword{interface}

\eof
\name{SCOMNamedFunctionClass}
\alias{SCOMNamedFunctionClass}
\title{Constructor for describing a named function COM server}
\description{
 This is a constructor function for creating 
\code{\link{SCOMNamedFunctionClass}} objects describing
a COM class that is implemented by named S functions.
}
\usage{
SCOMNamedFunctionClass(functions, name, ..., propertyNames = character(0), help = "", def = new("SCOMNamedFunctionClass"))
}
\arguments{
  \item{functions}{}
  \item{name}{the name of the COM class}
  \item{\dots}{arbitrary named values that are collected into a list
  and used as properties for each COM object of this class. These 
  properties are local to each COM object.}
  \item{propertyNames}{a character vector giving the names of S variables
   that are to be treated as properties for the COM objects of this class.
   Note that these are shared across all COM objects.}
  \item{help}{a string giving a brief description about this class that might
  be useful for a client.}
  \item{def}{an instance of the target S class used to represent this
   definition. Rarely specified directly but mainly used for inheritance.}
}

\value{
 An object of class \code{\link{SCOMNamedFunctionClass}}.
}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{SCOMNamedFunctionDispatch}}
 \code{\link{createCOMObject}}
}
\examples{

}
\keyword{interface}
\keyword{programming}

\eof
\name{computeCOMNames}
\alias{computeCOMNames}
\title{Generate name-id mapping}
\description{
 This constructs a mapping of the different names
 of the functions, properties and argument names within a
 collection of functions and property declarations for a COM object.
 The result is an integer vector that provides an ID for each 
 of the known names. This is used in the COM mechanism 
 to handle names in method/property  invocation.
}
\usage{
computeCOMNames(els)
}
\arguments{
  \item{els}{an list containing functions to export as methods 
  in a COM object and an 
  element named \code{.properties} giving the names of the
  properties to export.
 }
}
\details{
 This combines the property names with 
 the names of the methods and all of the 
 argument names for these functions.
}
\value{
 A named integer vector giving the mapping
 between the set of names to the corresponding integers.
}
\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/SWinRegistry}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{COMSIDispatchObject}}
}

\examples{

}
\keyword{interface}


\eof
\name{createCOMObject}
\alias{createCOMObject}
\title{Instantiate S COM Object}
\description{
 This generic function is called when the
 COM mechanism needs to create a COM object.
 This happens in two ways: a) when a client
 attempts to create a new instance of COM class,
 or b) when we convert an S value to a COM object
 as the return value or result from a COM method.
 The different methods for this generic take
 care of interpreting the S object suitably.
 In (a), typically the previously registered
 S description of the COM class is fetched
 from disk using the class UUID in the request
 and a suitable S object and dispatch mechanism
 are generated. 
 In case (b), we rely on there being a suitable
 method for \code{createCOMObject}.
 The default is to create a COM object using
 \code{\link{genericCOMHandler}}.

 Each method must return an S object that is
 simply an external pointer to a C++ object.
 The collection of available C++ classes
 is listed in the regular documentation for this package.
}
\usage{
createCOMObject(def)
}

\arguments{
  \item{def}{an S object either describing the COM class and 
  how to instantiate objects and control the dispatching of methods,
  or a regular S value that needs to be exposed as a dynamic COM object.}
}
\details{
 
}
\value{
 The methods must return an external pointer
 whose contents identify a C++ object that extends
 the \code{RCOMObject} class. 
 Typically, people will use the existing classes
 and modify the code provided in the examples and methods
 in this package. It is possible to introduce new C++ classes
 that control the dispatching in different ways or offer
 different interfaces. 
}
\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/SWinRegistry}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}

\seealso{
 \code{\link{genericCOMHandler}}
 \code{\link{COMSIDispatchObject}}
}

\examples{

}
\keyword{interface}

\eof
\name{genericCOMHandler}
\alias{genericCOMHandler}
\title{Dispatch mechanism for arbitrary S object}
\description{
 This function is used to create a dynamic dispatch handler
 for representing an arbitrary S object as  a COM object.
 It takes the S object and exports its named elements and
 attributes as properties and allows clients to call 
 arbitrary S functions as methods with this object as 
 the first argument.
}
\usage{
genericCOMHandler(obj)
}
\arguments{
  \item{obj}{the S object to be exported as a COM object.}
}
\details{
  This creates a closure that has access to the object
 and which provides methods for the \code{Invoke}
 and \code{GetIDsOfNames} methods for a COM object. 
 These are then passed to the C++ constructor function
 for the C++ class \code{RCOMSObject} which calls these methods
 when the corresponding COM object methods are invoked. 
}
\value{
 A list with two elements
 \item{Invoke}{the function which is called in response to 
  low-level COM invocations for this object.
  It is given the integer identifier for the method, 
  the style of invocation as a logical vector of length 4,
  the argument values, the integer identifiers for the names
  of any named arguments.
  The arguments are given in reverse order and the
  named arguments come first. See 
  \code{genericCOMHandler} for an example.
  }
 \item{GetIDsOfNames}{a function that maps the names of methods,
  properties and argument parameters to integers that are fixed across calls.
  This allows clients to cache these values.}
}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/RDCOMClient}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}

\examples{

}
\keyword{interface}

\eof
\name{getCOMClassDefFileName}
\alias{getCOMClassDefFileName}
\title{Find name of file containing centralized COM class definitions}
\description{
 When we register an S COM class definition, the default approach is to 
 store it in an S list indexed by its UUID and serialize this 
 list of know classes to a file. When R is asked to create an instance
 of such a class, it consults this list and finds the definition using
 the specified UUID.
 This function provides a way to find the name of the file containing
 the serialized list of definitions.
 To allow the user  provide some control over where this file is stored,
 we look first for an environment variable named
 \code{R_COM_CLASS_FILE}. If this is set, we assume its value is the name
 of the file.
 Otherwise, we check if there is a setting for the option named
 \code{RCOMClassFile}. If there is, we use that.
 And if neither of these is set, we use the default file name which is
 \code{RCOMClasses.rda} in the top directory of the installed
 RDCOMServer package.
}
\usage{
getCOMClassDefFileName()
}
\value{
 A string giving the default name of the file 
 to use for serializing and deserializing class definitions.
}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/SWinRegistry}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{registerCOMClass}}
 \code{\link{unregisterCOMClass}}
 \code{\link{GetCOMClassDef}}
 \code{\link{loadCOMClassDefs}}
 \code{\link{saveCOMClassDefs}}
}
\examples{
  getCOMClassDefFileName()

  options(RCOMClassFile="c:\\R\\COMClassDefs.rda")
  getCOMClassDefFileName()

  Sys.putenv("R_COM_CLASS_FILE" = "c:\\R\\COMClassDefs.rda")
  getCOMClassDefFileName()
}
\keyword{interface}


\eof
\name{isRegistered}
\alias{isRegistered}
\title{Determines if COM class is in registry}
\description{
This function checks the Windows registry
for an entry for the specified COM class.
The caller can identify the class either by
its user-level name or its UUID.
}
\usage{
isRegistered(name)
}
\arguments{
  \item{name}{an identifier for the COM class, which can be either
   the human readable name/program id, or the UUID for the class.}
}
\details{
 This looks in the standard place for the program name 
 and then looks for the class identifier.
}
\value{
 A logical value indicating if there is an entry in the
 windows registry associated with this DCOM class.
}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/SWinRegistry}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}
\seealso{
 \code{\link{lookupCLSIDByName}}
 \code{\link[SWinRegistry]{registryKeyExists}}
 \code{\link[SWinRegistry]{getRegistryValue}}
}

\examples{

}
\keyword{interface}


\eof
\name{lookupCLSIDByName}
\alias{lookupCLSIDByName}
\title{Resolve UUID for COM class}
\description{
This searches the Windows registry for 
the class identifier associated with
the given user-level name.
}
\usage{
lookupCLSIDByName(name)
}

\arguments{
  \item{name}{the user-level name (i.e. application identifier) for the COM class of interest.
   This is the name by which the class was registered.}
}
\details{
 This looks for an entry in the
 \code{HKEY_CLASSES_ROOT} key
 with this name and then looks for a
 sub-key named \code{CLSID}.
 The UUID is taken from the default value of this key.
}
\value{
 A
}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/SWinRegistry}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{SCOMIDispatch}}
 \code{\link{SCOMFunctionClass}}
}

\examples{

}
\keyword{interface}

\eof
\name{loadCOMClassDefs}
\alias{loadCOMClassDefs}
\alias{saveCOMClassDefs}
\title{Manages S definitions of COM classes}
\description{
 These functions provide the default mechanism
 for managing the global collection of definitions
 of COM classes exported from S.
 These organize a collection of class definitions
 indexed by the UUID of the class and store these
 in a file which is consulted when new COM objects
 are requested by clients.
}
\usage{
loadCOMClassDefs(fileName = getCOMClassDefFileName())
saveCOMClassDefs(fileName, COMDefs)
}
\arguments{
  \item{rda}{the name of an R data file containing the serialized object
    giving the list of COM class definitions. 
  }
  \item{fileName}{the name of the file in which the S object describing the class definitions
    is stored.}
  \item{COMDefs}{the collection of class defintions which is a named list 
  of class descriptions. The names/indices in this list are the UUIDs of the classes
  given as strings.}
}
\details{
 These functions merely serialize and deserialize an S object and 
 access elements within this object.
}
\value{
 \code{GetCOMClassDef} returns an object describing an S description
 of a COM class.
 \code{loadCOMClassDefs} returns the list of the COM  class definitions.
 \code{saveCOMClassDefs} returns the value \code{TRUE} if successful or
 raises an error  if not.
}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/SWinRegistry}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{getCOMClassDefFileName}}  
 \code{\link{registerCOMClass}}
 \code{\link{unregisterCOMClass}}
 \code{\link{createCOMObject}}
}

\examples{

}
\keyword{interface}

\eof
\name{matrixCOMHandler}
\alias{matrixCOMHandler}
\title{COM dispatcher for an S matrix}
\description{
 This is a function that generates suitable
 functions for making an S matrix accessible
 as a COM object.
 It returns a list of functions that have access 
 to the original matrix object and which allow
 client applications to access the values,
 dimensions, etc. of the matrix.
 This function is used when R tries to create
 a COM object when convering a matrix to a COM 
 return value or argument to a COM method. 
}
\usage{
matrixCOMHandler(m)
}
\arguments{
  \item{m}{the original S matrix that is to be expose
  to the COM world}
}
\details{
 This is called by the method for the \code{createCOMObject} 
 function for a matrix object.
 The resulting list of functions is then used
 to create a \code{COMSIDispatchObject} object
 which is passed to the C/C++ code to implement
 the COM object.
}
\value{
 A list of functions which act as methods for the COM
 representation of the matrix.
  \item{values}{return the elements of the matrix as a single array,
    using \code{\link[base]{as.vector}}}
  \item{dim}{return an array of length 2 giving the number of rows and columns of the matrix.}
  \item{column}{return a particular column. This can be called with a scalar or an array 
    identifying the column or columns. These identifiers may be numbers or strings.
    If more than one column is accessed, the result will be another COM  matrix object.}
  \item{row}{same as \code{column} above, but for rows.}
  \item{element}{access a particular element of the matrix, (i,j)}
  \item{nnrow}{the number of rows in the matrix}
  \item{ncol}{the number of columns in the matrix.}
  \item{rownames}{the array of row names, or NULL}
  \item{colnames}{the array of column names, or NULL}
  \item{.properties}{empty character vector.}
}

\seealso{
 \code{\link{createCOMObject}}
 \code{\link{COMSIDispatchObject}}
}
\examples{

}
\keyword{interface}

\eof
\name{registerCOMClass}
\alias{registerCOMClass}
\alias{unregisterCOMClass}
\title{Register S COM class definition}
\description{
  These functions manage the definitions of COM classes
 defined in R.  The idea is that the definitions are stored in an S list
 indexed by the class UUIDs. When a COM object is requested, R is started and then consults
 this list to find the actual definition. These functions also optionally modify the Windows registry to 
 explicitly make the COM class accessible to clients in other languages.
}
\usage{
registerCOMClass(def, clsid = def@classId, registry = TRUE, rda = getCOMClassDefFileName(), ...)
unregisterCOMClass(name, clsid = NULL, registry = TRUE,  rda = getCOMClassDefFileName())
}

\arguments{
  \item{def}{the S object representing the definition of the COM class within R.}
  \item{name}{the user-level name of the COM class. If clsid is not specified in 
    \code{unregisterCOMClass}, we lookup the ID using this name.}
  \item{clsid}{the UUID that is used to identify this COM class.
    This is used as the name in the R list and in the Windows registry.}
  \item{registry}{a logical value indicating whether to 
    also perform the operation in the Windows registry, making the
    class visible to other clients.}
  \item{rda}{a vector giving the name(s) of the file in which the 
    S COM class definitions are stored.  The first element is used as the
    location to find the current definitions and the second element is used
    as the file name in which to store the new definitions created when 
    adding or deleting this COM class. If the vector only has one element,
    that file name is used for both input and output.}
  \item{...}{arguments passed on to \code{\link{registerClassID}}}
}
\details{
 This reads the existing definitions from a serialized R object
 and adds or removes the class definition using the class ID (uuid)
 to identify it in the list.  If \code{registry} is \code{TRUE},
 the appropriate keys and values are added or removed from the
 Windows registry.
}
\value{
 For \code{registerClassID}, a logical value \code{TRUE} if successful 
 or an error is raised.

 \code{unregisterClassID} when called with \code{purge} 
 as \code{TRUE} returns the contents of the
 registry key for the UUID before it was removed.
 This allows one to restore any additional entries if one
 wants re-register the class.
 If \code{purge} is \code{FALSE},
 it will return \code{TRUE} if removing only the
 S-generated keys was successful,  or otherwise raise an error.
}

\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/SWinRegistry}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}


\seealso{
 \code{\link{registerClassID}}
 \code{\link{unregisterClassID}}
 \code{\link{getCOMClassDefFileName}}
}

\examples{
def <- SCOMFunctionClass(list(ttest = function(x, ...) t.test(as.numeric(x),...)), name = "R.TTest")
def@classId = getuuid("11D2A4BC-7EBC-4236-80FF-2588EFCD8821")
registerCOMClass(def)
}

\keyword{interface} 

\eof
\name{registerClassID}
\alias{registerClassID}
\alias{unregisterClassID}
\title{COM IDs for Windows Registry}
\description{
 These function manage the publication of a COM class 
 in the Windows registry.  
 \code{registerClassID} adds the relevant keys and values
 to the  Windows registry to make the COM class accessible
 by either human-readable name or class UUID.
 It associates the COM class with the R COM DLL or executable.
}
\usage{
registerClassID(name, clsid, progId = name, appId = NULL, version = "1.0", 
   dll = system.file("RDCOMEngine.dll", package = "RDCOMServer"), 
   exe = system.file("RDCOM.exe", package = "RDCOMServer"), 
   addClassID = FALSE, useEXE = FALSE, check = TRUE, rda = NULL, profile = NULL, ...,
   force = FALSE)
unregisterClassID(name, clsid = NULL, progId = name, purge = TRUE)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{name}{the name of the COM class by which people clients can create it.}
  \item{clsid}{the UUID that identifies the COM class.}
  \item{progId}{the human-readable program ID.
   When unregistering, this is 
   the name of the program identifier which may have been registerd
  in \code{registerCOMClass}. If specified and the key exists, this is removed.
  Not be implemented yet.}
  \item{appId}{}
  \item{version}{a string giving the version of the COM class}
  \item{dll}{the name of the DLL or executable (.EXE) file to use 
    when instantiating the COM object. This will almost always be either
    \code{RCOMEngine.dll} or \code{RDCOM.exe}
    for the in-process or out-of-process servers.
    This option can be controlled via the \code{useEXE} argument.
  }
  \item{exe}{the name of the EXE file to use for an out-of-process server.}
  \item{addClassID}{currently ignored!}
  \item{useEXE}{a logical value indicating whether to register the in-process DLL 
   or out-of-process EXE as the server code.}
  \item{check}{a logical value which controls whether we check to see if 
   there is alread an entry with the specified name or class identifier
   in the Windows registry.}
  \item{rda}{the name of a file which is expected to contain a
    serialized S object (created using \code{\link[base]{save}}).
    This is only used if we do not want to store the definition of
    the S object in the central S table of COM objects.
    This function then puts the name of the file (this value) 
    into the class's key in the Windows registry
    and S will look for this when attempting to create the object.
    One must be careful if using in-process servers on remote machines
    the file exists on the relevant machine.
    Otherwise one can use URIs.
   }
  \item{profile}{the name of an S source file or URI that is 
    \code{\link[base]{source}}'d when this class definition is
    first loaded. This can be used to initialize the R session
    appropriately.
    One must be careful if using in-process servers on remote machines
    the file exists on the relevant machine.
    Otherwise one can use URIs.
  }
  \item{...}{a collection of key name-value pairs of the form
       \code{key1=string, key2 = string, ...}
     These are added to the class id's registry folder/key
     as sub-keys. These can be used by methods 
     that operate on the objects or class identifier.
     The values can also be added to the S object defining the
     COM class and usually this is more convenient.
     However, it is easier to view them in the registry and 
     also to dynamically edit them without having to modify the
     S object.
   }
 \item{purge}{a logical value which if \code{TRUE} causes the entire key 
  and its sub-keys in the registry for this class to be deleted. 
  Any entries that were not added during the regular registration will
  be removed. A value of \code{FALSE} attempts to 
  remove only the keys that were registered by R in a call
  to \code{registerClassID}.}
 \item{force}{a logical value that, if \code{TRUE}, 
   proceeds with the registration even if there is an entry for that
   program ID with a different UUID for the class.}
}
\details{
 This uses the \code{SWinRegistry} package
 to create or delete the keys in the Windows 
 registry.
}
\value{
 If successful, this returns the class identifier 
 as a named string (character vector). The name is
 the name supplied. This is useful if one allows the function to 
 generate the UUID.
}
\references{
 \url{http://www.omegahat.org/RDCOMServer}
 \url{http://www.omegahat.org/SWinRegistry}
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}

\seealso{
 \code{\link{registerCOMClass}}
 \code{\link{unregisterCOMClass}}
}

\examples{

}
\keyword{interface}


\eof
